// BlinkGame - Hunt for the Lost Pixels
// an SDL game in honor of the BlinkenArea crew
//
// (C) 2005-06 Rene Schickbauer
//
// See License.txt for licensing information
//
#ifdef NOP_CODE
/*--------------------------------------------------------------------
					CVS AUTO-GENERATED INFO
$Author: cavac $
$Date: 2006/03/02 13:03:43 $
$Name: rel-0-3-patches $
$Revision: 1.4 $
$State: Exp $

$Log: monstersprites.h,v $
Revision 1.4  2006/03/02 13:03:43  cavac
Re-formatted all .cpp and .h files a bit to get rid of the TAB/SPACE mess
that makes troubles with XCode's "re-format" feature

Revision 1.3  2006/02/12 15:54:11  cavac
Added a new monster: The Buggy
This monster patrols a tight space except when the player is in sight

Revision 1.2  2006/01/09 00:50:34  cavac
Bugfixed CVS autogenerated entries in background.cpp
Added CVS autogenerated entries to remaining .cpp and .h files
Updated copyright from "2005 Rene Schickbauer" to "2005-06 Rene Schickbauer"


--------------------------------------------------------------------*/
#endif //NOP_CODE

#ifndef MONSTERSPRITES_H
#define MONSTERSPRITES_H

#include "globals.h"

struct MONSTERAI {
	Uint32 lastTurnTick;
};

struct MONSTERS {
	Uint32 type;
	Uint32 startx;
	Uint32 starty;
	double monsterx;
	double monstery;
	double monstervx;
	double monstervy;
	double maxspeed;
	bool isAlive;
	bool isDying;
	Uint32 dyeCount;
	Uint32 score;
	Uint32 spritenum;
	Uint32 spritetick;
	MOVE_TYPE mtype;
	MOVE_TYPE newmtype;
	MONSTERAI ai;
	MONSTERS* next;
};


void loadMonsterSprites();
void unloadMonsterSprites();
void addMonster(Uint32 type, Uint32 x, Uint32 y);
void initMonsters();
void freeMonsters();
void blitMonsterSprite(Sint32 x, Sint32 y, SDL_Surface* src, Uint32 srcoffs, Uint32 maxheight);
void showMonsterSprites(Uint32 screenXoffs, Uint32 screenYoffs);
void monsterPhysics(Uint32 playerx, Uint32 playery);

#endif // MONSTERSPRITES_H

