// BlinkGame - Hunt for the Lost Pixels
// an SDL game in honor of the BlinkenArea crew
//
// (C) 2005-06 Rene Schickbauer
//
// See License.txt for licensing information
//
#ifdef NOP_CODE
/*--------------------------------------------------------------------
					CVS AUTO-GENERATED INFO
$Author: cavac $
$Date: 2006/03/02 13:03:41 $
$Name: rel-0-3-patches $
$Revision: 1.2 $
$State: Exp $

$Log: actioncapture.h,v $
Revision 1.2  2006/03/02 13:03:41  cavac
Re-formatted all .cpp and .h files a bit to get rid of the TAB/SPACE mess
that makes troubles with XCode's "re-format" feature

Revision 1.1  2006/02/10 17:50:26  cavac
Added ActionCapture feature (define ALLOW_ACTIONCAPTURE to allow this
resource-hungry extra)

--------------------------------------------------------------------*/
#endif //NOP_CODE

#ifdef ALLOW_ACTIONCAPTURE

#ifndef ACTIONCAPTURE_H
#define ACTIONCAPTURE_H

void initActionCapture();
void startActionCapture();
void stopActionCapture();
void renderActionCapture(Uint32 spriteoffsx, Uint32 spriteoffsy);
void deInitActionCapture();

#endif // ACTIONCAPTURE_H

#endif // ALLOW_ACTIONCAPTURE
