// BlinkGame - Hunt for the Lost Pixels
// an SDL game in honor of the BlinkenArea crew
//
// (C) 2005-06 Rene Schickbauer
//
// See License.txt for licensing information
//
#ifdef NOP_CODE
/*--------------------------------------------------------------------
					CVS AUTO-GENERATED INFO
$Author: cavac $
$Date: 2006/03/02 13:03:42 $
$Name: rel-0-3-patches $
$Revision: 1.4 $
$State: Exp $

$Log: errorcodes.h,v $
Revision 1.4  2006/03/02 13:03:42  cavac
Re-formatted all .cpp and .h files a bit to get rid of the TAB/SPACE mess
that makes troubles with XCode's "re-format" feature

Revision 1.3  2006/02/26 14:06:04  cavac
Added menu support

Revision 1.2  2006/02/24 18:48:26  cavac
Finally added real scripting support
Added Testlevel (level0) which can be enabled from globals.h

Revision 1.1  2006/02/17 10:49:42  cavac
Added error-handling

--------------------------------------------------------------------*/
#endif //NOP_CODE

#ifdef PARSE_ERRORCODES
// The following lines are used via macros from errorhandler.*
// DO NOT INCLUDE THEM ELSEWHERE, INCLUDE errorhandler.h
XX(ERROR_FILE_READ,	 100, "Can't open file for reading")
XX(ERROR_FILE_WRITE,	101, "Can't open file for writing")
XX(ERROR_IMAGE_READ,	102, "Can't read or understand image file")
XX(ERROR_SOUND_READ,	103, "Can't read or understand sound file")
XX(ERROR_BOUNDARY,	  104, "Index out of boundary")
XX(ERROR_INDEX_INVALID, 105, "Invalid index")
XX(ERROR_POSITION,	  106, "Illegal Position")
XX(ERROR_LUAINIT,	   107, "Could not initialize LUA")
XX(ERROR_LUALOAD,	   108, "Could not load LUA file")
XX(ERROR_LUAPARSE,	  109, "Could not parse LUA file")
XX(ERROR_LUACALL,	   110, "Error running LUA function")
XX(ERROR_LUANOINIT,	 111, "LUA not initialized")
XX(ERROR_LUAPARAMFORMAT,112, "Unknown LUA parameter format")
XX(ERROR_LUAARGCOUNT,   113, "Too many arguments for LUA stack buffer")
XX(ERROR_LUARESULTTYPE, 114, "Wrong result type from LUA")
XX(ERROR_LUAL2BTYPE,	115, "Wrong type from LUA call to BlinkenSisters")
XX(ERROR_LUAL2BDIE,	 116, "DIE called from LUA script")
XX(ERROR_FONTLOAD,	  117, "Could not load menu font")
#endif // PARSE_ERRORCODES
