// BlinkGame - Hunt for the Lost Pixels
// an SDL game in honor of the BlinkenArea crew
//
// (C) 2005-06 Rene Schickbauer
//
// See License.txt for licensing information
//
#ifdef NOP_CODE
/*--------------------------------------------------------------------
					CVS AUTO-GENERATED INFO
$Author: cavac $
$Date: 2006/03/02 13:03:43 $
$Name: rel-0-3-patches $
$Revision: 1.5 $
$State: Exp $

$Log: osdef.cpp,v $
Revision 1.5  2006/03/02 13:03:43  cavac
Re-formatted all .cpp and .h files a bit to get rid of the TAB/SPACE mess
that makes troubles with XCode's "re-format" feature

Revision 1.4  2006/01/25 19:50:20  cavac
Cleaned up OS dependant code (osdef.*) and temporarily disabled OS_deps.pl

Revision 1.3  2006/01/23 18:47:55  cavac
Patches for unix make system for building on Linux  and other
   Unix systems (thanks to: Dauti)
Tests for new colission detection (default: Use old one)
Added Joystick/Gamepad support (only tested with a cheap logitech gamepad)
Fixed some warnings on linux/gcc3
Added true-type fonts for planned menu system
Added library SDL_ttf in xcode for planned menu-system

Revision 1.2  2006/01/18 01:22:39  cavac
Added SDL_Image to load additional graphics formats
Updated osdef* and xcode-project to to add SDL_Image
Updated all image-loader functions to use SDL_Image
Added level6 as temporary testlevel
Added code to display start- and goal-points with an animation (needs some fixing)
Removed temporary "marker"-tiles from level-definitions, as we have now an
animated display for that.

Revision 1.1  2006/01/09 13:00:29  cavac
Added Project-files for VisualC++ 6 (VC6)
Added needed SDL libraries for WIN32
Ported Code to compile on Win32 (some problems remain)
TODO: Add OGG-Support for Win32 (libogg + libvorbis)
Added Installer-Script for Inno-Setup (Win32)

--------------------------------------------------------------------*/
#endif //NOP_CODE

#include "osdef.h"

// OS-dependant code goes here
