// BlinkGame - Hunt for the Lost Pixels
// an SDL game in honor of the BlinkenArea crew
//
// (C) 2005-06 Rene Schickbauer
//
// See License.txt for licensing information
//
#ifdef NOP_CODE
/*--------------------------------------------------------------------
					CVS AUTO-GENERATED INFO
$Author: cavac $
$Date: 2006/03/02 13:03:42 $
$Name: rel-0-3-patches $
$Revision: 1.2 $
$State: Exp $

$Log: errorhandler.h,v $
Revision 1.2  2006/03/02 13:03:42  cavac
Re-formatted all .cpp and .h files a bit to get rid of the TAB/SPACE mess
that makes troubles with XCode's "re-format" feature

Revision 1.1  2006/02/17 10:49:42  cavac
Added error-handling

--------------------------------------------------------------------*/
#endif //NOP_CODE

#ifndef ERRORHANDLER_H
#define ERRORHANDLER_H

#include "globals.h"

#define PARSE_ERRORCODES
#define XX(errenum, errcode, errtxt) errenum = errcode,
typedef enum _ERRORCODES {
#include "errorcodes.h"
	ERROR_LASTERROR
} ERRORCODES;
#undef XX
#undef PARSE_ERRORCODES

char* getErrorText(Uint32 errorcode);

#define DIE(err, txt) dieWithError(err, txt, __LINE__, __FILE__);
void dieWithError(Uint32 errorcode, char* extrainfo, Uint32 linenum, char* filename);

#endif // ERRORHANDLER_H
