// BlinkGame - Hunt for the Lost Pixels
// an SDL game in honor of the BlinkenArea crew
//
// (C) 2005-06 Rene Schickbauer
//
// See License.txt for licensing information
//
#ifdef NOP_CODE
/*--------------------------------------------------------------------
					CVS AUTO-GENERATED INFO
$Author: cavac $
$Date: 2006/03/02 13:03:42 $
$Name: rel-0-3-patches $
$Revision: 1.5 $
$State: Exp $

$Log: memops.cpp,v $
Revision 1.5  2006/03/02 13:03:42  cavac
Re-formatted all .cpp and .h files a bit to get rid of the TAB/SPACE mess
that makes troubles with XCode's "re-format" feature

Revision 1.4  2006/02/12 20:39:03  cavac
Added a newline to fix compiler warning

Revision 1.3  2006/02/10 17:50:26  cavac
Added ActionCapture feature (define ALLOW_ACTIONCAPTURE to allow this
resource-hungry extra)

Revision 1.2  2006/01/09 00:50:34  cavac
Bugfixed CVS autogenerated entries in background.cpp
Added CVS autogenerated entries to remaining .cpp and .h files
Updated copyright from "2005 Rene Schickbauer" to "2005-06 Rene Schickbauer"


--------------------------------------------------------------------*/
#endif //NOP_CODE

#include "globals.h"
#include "memops.h"

fastcopy *fc_src;
fastcopy *fc_dest;

#ifdef ALLOW_ACTIONCAPTURE
acCapFastcopy *acc_fc_src;
acCapFastcopy *acc_fc_dest;
#endif // ALLOW_ACTIONCAPTURE

