// BlinkGame - Hunt for the Lost Pixels
// an SDL game in honor of the BlinkenArea crew
//
// (C) 2005-06 Rene Schickbauer
//
// See License.txt for licensing information
//
#ifdef NOP_CODE
/*--------------------------------------------------------------------
					CVS AUTO-GENERATED INFO
$Author: cavac $
$Date: 2006/03/02 13:03:43 $
$Name: rel-0-3-patches $
$Revision: 1.11 $
$State: Exp $

$Log: osdef.h,v $
Revision 1.11  2006/03/02 13:03:43  cavac
Re-formatted all .cpp and .h files a bit to get rid of the TAB/SPACE mess
that makes troubles with XCode's "re-format" feature

Revision 1.10  2006/02/24 18:48:26  cavac
Finally added real scripting support
Added Testlevel (level0) which can be enabled from globals.h

Revision 1.9  2006/02/13 22:05:12  dauti
(hopefully) fixed the round()-Problem in various compilers/operating systems:

The round() - function is defined in the C99 Standard, floor in an older standard (C89)
(see e.g. http://www.schweikhardt.net/identifiers.html)
So if the compiler does not support C99, #define round(x) floor((x)+0.5)

Revision 1.8  2006/02/02 08:58:05  cavac
Added Level 6 (previous level6 was only a test-level)

Revision 1.7  2006/01/27 17:04:02  cavac
Updated VC6 workspace to current release
Added ogg/vorbis support for Win32
Added SDL_Image and SDL_ttf support
Changed build- and installer-system for easier maintenance

Revision 1.6  2006/01/25 19:58:06  cavac
Ooops. osdef.h shouldn't include itself...

Revision 1.5  2006/01/25 19:50:20  cavac
Cleaned up OS dependant code (osdef.*) and temporarily disabled OS_deps.pl


--------------------------------------------------------------------*/
#endif //NOP_CODE

#ifndef OSDEPS_H
#define OSDEPS_H
#include <SDL.h>
#include <SDL_mixer.h>
#include <SDL_image.h>

// (hopefully) fixed the round()-Problem in various operating systems:
// 
// The round() - function is defined in the C99 Standard, floor in an older standard (C89)
// (see e.g. http://www.schweikhardt.net/identifiers.html)
// So if the compiler does not support C99, #define round(x) floor((x)+0.5)
// 
// The following C99-related defines are defined (in gcc):
// gcc  -E -dM test.cpp | grep C99
// #define __USING_NAMESPACE_C99(name)
// #define __USE_ISOC99 1
// #define _ISOC99_SOURCE 1
// #define __END_NAMESPACE_C99
// #define __BEGIN_NAMESPACE_C99
//
// Which define is the 'best one' (defined in most compilers?)

#ifndef __USE_ISOC99
#define round(x) floor((x)+0.5)
#endif

#ifdef __WIN32__
#define RESPATH ""
#endif // __WIN32__

#ifdef XCODE_DEVELOPMENT
#undef CAVAC_RELEASEMODE
#undef ENABLE_FULLSCREEN
#endif // XCODE_DEVELOPMENT

#endif // OSDEPS_H
