// BlinkGame - Hunt for the Lost Pixels
// an SDL game in honor of the BlinkenArea crew
//
// (C) 2005-06 Rene Schickbauer
//
// See License.txt for licensing information
//
#ifdef NOP_CODE
/*--------------------------------------------------------------------
					CVS AUTO-GENERATED INFO
$Author: dauti $
$Date: 2006/03/16 20:37:17 $
$Name: rel-0-3-patches $
$Revision: 1.4 $
$State: Exp $

$Log: cmdopts.h,v $
Revision 1.4  2006/03/16 20:37:17  dauti
Include the name of the called executable in the help text. Just "blinkensisters" was wrong.

Revision 1.3  2006/03/02 13:03:41  cavac
Re-formatted all .cpp and .h files a bit to get rid of the TAB/SPACE mess
that makes troubles with XCode's "re-format" feature

Revision 1.2  2006/02/13 09:41:54  cavac
Fixed compiler warning (Tracker #1430545)

Revision 1.1  2006/02/06 11:14:37  cavac
Module for handling command-line options


--------------------------------------------------------------------*/
#endif //NOP_CODE

#ifndef CMDOPTS_H
#define CMDOPTS_H

bool initCmdOpt(int ARGC, char **ARGV);
bool getCmdOptBool(char* name);
char* getCmdOptString(char* name);
void helpCmdOpt(char* name);

struct CMD_OPTS {
	char shortname[255];
	char fullname[255];
	char description[255];
	bool isBool;
	bool defaultBool;
	char defaultString[255];
};

//extern CMD_OPTS cmd_opt_data;

#endif // CMDOPTS_H
