// BlinkGame - Hunt for the Lost Pixels
// an SDL game in honor of the BlinkenArea crew
//
// (C) 2005-06 Rene Schickbauer
//
// See License.txt for licensing information
//
#ifdef NOP_CODE
/*--------------------------------------------------------------------
CVS AUTO-GENERATED INFO
$Author: cavac $
$Date: 2006/02/24 18:48:26 $
$Name: rel-0-3-patches $
$Revision: 1.3 $
$State: Exp $

$Log: joystick.h,v $
Revision 1.3  2006/02/24 18:48:26  cavac
Finally added real scripting support
Added Testlevel (level0) which can be enabled from globals.h

Revision 1.2  2006/02/12 20:38:18  cavac
Removed warning "declaration requires a typedef name"

Revision 1.1  2006/01/23 18:47:55  cavac
Patches for unix make system for building on Linux  and other
   Unix systems (thanks to: Dauti)
Tests for new colission detection (default: Use old one)
Added Joystick/Gamepad support (only tested with a cheap logitech gamepad)
Fixed some warnings on linux/gcc3
Added true-type fonts for planned menu system
Added library SDL_ttf in xcode for planned menu-system

--------------------------------------------------------------------*/
#endif //NOP_CODE

#ifndef JOYSTICK_H
#define JOYSTICK_H

typedef enum _JOYSTICK_MOVE {
	JOYSTICK_NONE = 0,
	JOYSTICK_LEFT = 1,
	JOYSTICK_RIGHT = 2,
	JOYSTICK_UP = 4,
	JOYSTICK_DOWN = 8,
	JOYSTICK_JUMP = 16,
	JOYSTICK_ACTION = 32
} JOYSTICK_MOVE;
	
void initJoystick();
void deInitJoystick();
Uint32 getJoystickMoves();

#endif // JOYSTICK_H
