#!/usr/bin/perl


# BlinkGame - Hunt for the Lost Pixels
# an SDL game in honor of the BlinkenArea crew
#
# (C) 2005-06 Rene Schickbauer
#
# See License.txt for licensing information
#

use strict;
use warnings;

my $fnames = " ";

for(my $i = 0; $i < 200; $i++) {
    $fnames .= " ACTIONCAPTURE_$i.bmp";
}

#my $cmd = "convert -shave 200x100 -adjoin -delay 10 $fnames anim.gif";
#my $cmd = "convert -crop 240x280+200+100! -adjoin -delay 10 $fnames anim.gif";
my $cmd = "convert -adjoin -delay 20 $fnames anim.gif";

print "Converting images to sequence...\n";
`$cmd`;
print "...done\n";
