// BlinkGame - Hunt for the Lost Pixels
// an SDL game in honor of the BlinkenArea crew
//
// (C) 2005-06 Rene Schickbauer
//
// See License.txt for licensing information
//
#ifdef NOP_CODE
/*--------------------------------------------------------------------
					CVS AUTO-GENERATED INFO
$Author: cavac $
$Date: 2006/03/02 13:03:41 $
$Name: rel-0-3-patches $
$Revision: 1.2 $
$State: Exp $

$Log: bl_lua_bindings.h,v $
Revision 1.2  2006/03/02 13:03:41  cavac
Re-formatted all .cpp and .h files a bit to get rid of the TAB/SPACE mess
that makes troubles with XCode's "re-format" feature

Revision 1.1  2006/02/24 18:48:26  cavac
Finally added real scripting support
Added Testlevel (level0) which can be enabled from globals.h

--------------------------------------------------------------------*/
#endif //NOP_CODE

#ifndef BL_LUA_BINDINGS_H
#define BL_LUA_BINDINGS_H

#include "lua.h"
#include "lualib.h"

void blRegisterLuaBindings(lua_State *L);

#endif // BL_LUA_BINDINGS_H
