#!/usr/bin/perl

use strict;
use warnings;

my @files = `find . -name '*.c'`;

foreach my $fname (@files) {
    $fname =~ s/\r//g;
    chomp $fname;

    my $newfname = $fname . "pp";

    print "$fname -> $newfname\n";

    `mv $fname $newfname`;
}
