// BlinkGame - Hunt for the Lost Pixels
// an SDL game in honor of the BlinkenArea crew
//
// (C) 2005-06 Rene Schickbauer
//
// See License.txt for licensing information
//
#ifdef NOP_CODE
/*--------------------------------------------------------------------
					CVS AUTO-GENERATED INFO
$Author: cavac $
$Date: 2006/03/02 13:03:43 $
$Name: rel-0-3-patches $
$Revision: 1.6 $
$State: Exp $

$Log: tiles.h,v $
Revision 1.6  2006/03/02 13:03:43  cavac
Re-formatted all .cpp and .h files a bit to get rid of the TAB/SPACE mess
that makes troubles with XCode's "re-format" feature

Revision 1.5  2006/02/10 18:56:17  cavac
Code Audit: Fixed many 64Bit based problems (mostly untested)

Revision 1.4  2006/01/19 19:19:15  cavac
Various bugfixes and performance tunings

Revision 1.3  2006/01/18 01:22:39  cavac
Added SDL_Image to load additional graphics formats
Updated osdef* and xcode-project to to add SDL_Image
Updated all image-loader functions to use SDL_Image
Added level6 as temporary testlevel
Added code to display start- and goal-points with an animation (needs some fixing)
Removed temporary "marker"-tiles from level-definitions, as we have now an
animated display for that.

Revision 1.2  2006/01/09 00:50:34  cavac
Bugfixed CVS autogenerated entries in background.cpp
Added CVS autogenerated entries to remaining .cpp and .h files
Updated copyright from "2005 Rene Schickbauer" to "2005-06 Rene Schickbauer"


--------------------------------------------------------------------*/
#endif //NOP_CODE

#ifndef TILES_H
#define TILES_H

void initTiles(char *fname);
void deInitTiles();
void paintSingleTile(Uint32 num, int x, int y);
Uint32 getNumOfTiles();
void paintLevelTiles(Uint32 xoffs, Uint32 yoffs);
void paintSpecialTiles(Uint32 xoffs, Uint32 yoffs, bool canExit);

typedef enum _SPECIALTILES {
	SPECIALTILE_START,
	SPECIALTILE_GOAL
} SPECIALTILES;

#endif // TILES_H
