// BlinkGame - Hunt for the Lost Pixels
// an SDL game in honor of the BlinkenArea crew
//
// (C) 2005-06 Rene Schickbauer
//
// See License.txt for licensing information
//
#ifdef NOP_CODE
/*--------------------------------------------------------------------
					CVS AUTO-GENERATED INFO
$Author: cavac $
$Date: 2006/03/02 13:03:41 $
$Name: rel-0-3-patches $
$Revision: 1.3 $
$State: Exp $

$Log: colission.h,v $
Revision 1.3  2006/03/02 13:03:41  cavac
Re-formatted all .cpp and .h files a bit to get rid of the TAB/SPACE mess
that makes troubles with XCode's "re-format" feature

Revision 1.2  2006/01/09 00:50:34  cavac
Bugfixed CVS autogenerated entries in background.cpp
Added CVS autogenerated entries to remaining .cpp and .h files
Updated copyright from "2005 Rene Schickbauer" to "2005-06 Rene Schickbauer"


--------------------------------------------------------------------*/
#endif //NOP_CODE

#ifndef COLISSION_H
#define COLISSION_H

#include "monstersprites.h"

typedef enum _COL_TYPE {
	COL_NONE = 0,
	COL_LEFT = 1,
	COL_RIGHT = 2,
	COL_UP = 4,
	COL_DOWN = 8
} COL_TYPE;

Uint32 getTilesColission(Uint32 playerx, Uint32 playery);
Uint32 getTilesMonsterColission(MONSTERS* monster);
Uint32 getPlayerMonsterColission(Uint32 playerx, Uint32 playery);
void handlePixelCollission(Uint32 playerx, Uint32 playery);

#endif // COLISSION_H
