// BlinkGame - Hunt for the Lost Pixels
// an SDL game in honor of the BlinkenArea crew
//
// (C) 2005-06 Rene Schickbauer
//
// See License.txt for licensing information
//
#ifdef NOP_CODE
/*--------------------------------------------------------------------
					CVS AUTO-GENERATED INFO
$Author: cavac $
$Date: 2006/03/02 13:03:43 $
$Name: rel-0-3-patches $
$Revision: 1.6 $
$State: Exp $

$Log: sound.h,v $
Revision 1.6  2006/03/02 13:03:43  cavac
Re-formatted all .cpp and .h files a bit to get rid of the TAB/SPACE mess
that makes troubles with XCode's "re-format" feature

Revision 1.5  2006/02/24 20:25:39  cavac
Added sound-support to scripting engine

Revision 1.4  2006/02/17 19:07:59  cavac
Added rudimentary sound effects (FX) support

Revision 1.3  2006/01/23 18:47:55  cavac
Patches for unix make system for building on Linux  and other
   Unix systems (thanks to: Dauti)
Tests for new colission detection (default: Use old one)
Added Joystick/Gamepad support (only tested with a cheap logitech gamepad)
Fixed some warnings on linux/gcc3
Added true-type fonts for planned menu system
Added library SDL_ttf in xcode for planned menu-system

Revision 1.2  2006/01/09 00:50:34  cavac
Bugfixed CVS autogenerated entries in background.cpp
Added CVS autogenerated entries to remaining .cpp and .h files
Updated copyright from "2005 Rene Schickbauer" to "2005-06 Rene Schickbauer"


--------------------------------------------------------------------*/
#endif //NOP_CODE

#ifndef SOUND_H
#define SOUND_H

typedef enum _FX_SOUNDS {
	FX_COLLECT_PIXEL = 0,
	FX_KILL_MONSTER,
	FX_KILL_PLAYER,
	FX_LEVEL_FINISHED,
	FX_MAX_PREDEF
} FX_SOUNDS;

void initSound();
void soundStartMusic(char* fname);
void soundStopMusic();
void soundPlayFX(Uint32 fx);
void deInitSound();

Uint32 addSoundFXLua(char* fname);
void soundPlayFXLua(Uint32 fx);
void initSoundFXLua();
void deInitSoundFXLua();



#endif // SOUND_H
