// BlinkGame - Hunt for the Lost Pixels
// an SDL game in honor of the BlinkenArea crew
//
// (C) 2005-06 Rene Schickbauer
//
// See License.txt for licensing information
//
#ifdef NOP_CODE
/*--------------------------------------------------------------------
					CVS AUTO-GENERATED INFO
$Author: cavac $
$Date: 2006/03/02 13:03:42 $
$Name: rel-0-3-patches $
$Revision: 1.3 $
$State: Exp $

$Log: fonthandler.h,v $
Revision 1.3  2006/03/02 13:03:42  cavac
Re-formatted all .cpp and .h files a bit to get rid of the TAB/SPACE mess
that makes troubles with XCode's "re-format" feature

Revision 1.2  2006/02/27 16:37:21  dauti
Generalized renderFontHandlerText(), so that one can specify the color of the text.

Revision 1.1  2006/02/26 14:06:05  cavac
Added menu support

--------------------------------------------------------------------*/
#endif //NOP_CODE

#ifndef FONTHANDLER_H
#define FONTHANDLER_H

void initFontHandler();
void deInitFontHandler();
void renderFontHandlerText(Sint32 x, Sint32 y, char* text, SDL_Color fontcolor, bool hcentered, bool vcentered);

#endif // FONTHANDLER_H
